/*
 * DiscreteValidator.hpp
 *
 *  Created on: Apr 13, 2012
 *      Author: ankele
 */

#ifndef DISCRETEVALIDATOR_HPP_
#define DISCRETEVALIDATOR_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Validator.hpp"

#include <vector>

/** A validator with a discrete list of valid values.
 *
 */
template <class T>
class DiscreteValidator : public Validator<T>
{
public:
  DiscreteValidator(const std::vector<T> &_ValidValues) : ValidValues(_ValidValues) {};
  virtual ~DiscreteValidator() {};

  bool isValid(const T & _value) const
  {
    typename std::vector<T>::const_iterator iter = std::find(ValidValues.begin(), ValidValues.end(), _value);
    if (iter != ValidValues.end()) {
      //std::cout << "Found " << _value << ":" << *iter << std::endl;
      return true;
    } else {
      //std::cout << "Did not find " << _value << "." << std::endl;
      return false;
    }
  }

  Validator<T>* clone() const { return new DiscreteValidator<T>(ValidValues); };

  // comparator
  bool operator==(const Validator<T> &_instance) const
  {
    const DiscreteValidator<T> *inst = dynamic_cast<const DiscreteValidator<T> *>(&_instance);
    if (inst)
      return ValidValues == inst->ValidValues;
    return false;
  };

  void appendValidValue(const T &_value)
  {
    ASSERT(!isValid(_value),
        "DiscreteValidator<>::appendValidValue() - value "+toString(_value)+" is already among the valid");
    ValidValues.push_back(_value);
  }

private:
  //!> list of valid values
  std::vector<T> ValidValues;
};


#endif /* DISCRETEVALIDATOR_HPP_ */
